from __future__ import absolute_import, unicode_literals

from collections import defaultdict
import logging
import subprocess

from wheel.install import WheelFile, parse_version

log = logging.getLogger(__name__)


def call_pips(config, pip_args):
    pip_opts = ['-q', 'wheel', '--wheel-dir', str(config.wheelhouse_dpath), '--use-wheel',
                '--find-links', str(config.wheelhouse_dpath)]
    if config.verbose:
        pip_opts.pop(0)

    for pip_bin in config.pip_bins:
        all_args = [pip_bin] + pip_opts + pip_args
        log.info('Call %s', all_args)
        return_code = subprocess.call(all_args)
        if return_code != 0:
            # we had an error, don't try to finish
            return


def build_files(config):
    for req_file in config.requirement_files:
        return_code = call_pips(config, ['-r', str(req_file)])
        if return_code != 0:
            # we had an error, don't try to finish
            return


def build_packages(config, packages):
    packages = config.alias_sub(packages)
    call_pips(config, packages)


def prune_list(config):
    newest = {}
    older = list()
    wheel_files = config.wheelhouse_dpath.glob('*.whl')
    for wheel_fpath in wheel_files:
        wheel_name = wheel_fpath.name
        parts = wheel_name.split('-')
        distname, version = parts[0:2]
        wheel_key = '{}-{}'.format(distname, '-'.join(parts[2:]))
        version = parse_version(version)
        if wheel_key not in newest:
            newest[wheel_key] = (version, wheel_fpath)
        else:
            newest_version, newest_wheel_fpath = newest[wheel_key]
            if version < newest_version:
                older.append(wheel_fpath)
            else:
                older.append(newest_wheel_fpath)
    return older




import ConfigParser

from pathlib import Path

from .appdirs import AppDirs

appdirs = AppDirs('wheelhouse', appauthor=False, multipath=True)


class Config(object):
    config_fname = 'wheelhouse.ini'

    def __init__(self, verbose):
        self.cp = None
        self.found_fpaths = None
        self.project_root_dpath = None
        self.verbose = verbose
        self.set_project_root()
        self.load_files()

    def search_fpaths(self):
        """
            Return a platform dependent list of file paths which represent locations that a
            wheelhouse.ini file might exist.

            Configs are looked for:

            1) in a user-specific file
            2) a file in the current path and every directory above until we hit the root dir.

            Configs are cumulative with more specific configuration taking precidence over
            earlier configs read.
        """
        dpaths = [Path(appdirs.user_config_dir) / self.config_fname] + self.walk_paths()
        return map(str, dpaths)

    def walk_paths(self):
        walked_paths = []
        config_dpath = Path.cwd()
        path_root = config_dpath.root
        walked_paths.append(config_dpath)
        while True:
            config_dpath = config_dpath.parent
            walked_paths.append(config_dpath)
            if str(config_dpath) == path_root:
                break

        return [dpath.joinpath(self.config_fname) for dpath in walked_paths]

    def set_project_root(self):
        for fpath in self.walk_paths():
            if fpath.exists():
                self.project_root_dpath = fpath.resolve().parent
                break

    def load_files(self):
        self.cp = ConfigParser.SafeConfigParser()
        self.set_defaults()
        self.found_fpaths = self.cp.read(self.search_fpaths())

    def set_defaults(self):
        self.cp.add_section('aliases')
        self.cp.add_section('wheelhouse')
        self.cp.set('wheelhouse', 'requirement_files', '')
        self.cp.set('wheelhouse', 'requirements_path', 'requirements')
        self.cp.set('wheelhouse', 'wheelhouse_path', 'requirements/wheelhouse')
        self.cp.set('wheelhouse', 'pip_bins', 'pip')

    @property
    def requirements_dpath(self):
        return self.project_root_dpath / self.cp.get('wheelhouse', 'requirements_path')

    @property
    def wheelhouse_dpath(self):
        return self.project_root_dpath / self.cp.get('wheelhouse', 'wheelhouse_path')

    @property
    def requirement_files(self):
        retval = []
        for fname in self.cp.get('wheelhouse', 'requirement_files').strip().splitlines():
            retval.append(self.requirements_dpath / fname)
        return retval

    @property
    def pip_bins(self):
        return map(lambda x: x.strip(), self.cp.get('wheelhouse', 'pip_bins').strip().split(','))

    def alias_sub(self, package_names):
        retval = []
        for name in package_names:
            try:
                retval.append(self.cp.get('aliases', name))
            except ConfigParser.NoOptionError:
                retval.append(name)
        return retval



import logging

import click

from .config import Config
from wheelhouse import core

VERBOSE = False


@click.group()
@click.option('-v', '--verbose', is_flag=True)
def wheelhouse(verbose):
    global VERBOSE
    VERBOSE = verbose
    logging_level = logging.INFO if verbose else logging.WARN
    logging.basicConfig(level=logging_level)


@wheelhouse.command()
@click.argument('packages', nargs=-1)
def build(packages):
    config = Config(verbose=VERBOSE)
    if not packages:
        core.build_files(config)
    else:
        core.build_packages(config, packages)


@wheelhouse.command()
def config():
    config = Config(verbose=VERBOSE)

    click.echo('Config search paths searched:')
    for val in config.search_fpaths():
        click.echo('    {}'.format(val))

    click.echo('Config files used (higher precidence last):')
    for val in config.found_fpaths:
        click.echo('    {}'.format(val))

    click.echo('Config values:')
    if config.project_root_dpath is None:
        click.echo('    Error: no project wheelhouse.ini file found')
    else:
        click.echo('    project root = {}'.format(config.project_root_dpath))
        click.echo('    requirements root = {}'.format(config.requirements_dpath))
        click.echo('    wheelhouse root = {}'.format(config.wheelhouse_dpath))

        click.echo('    [wheelhouse]')
        click.echo('        requirement_files = {}'.format(config.requirement_files))
        click.echo('        pip_bins = {}'.format(config.pip_bins))


@wheelhouse.command()
def prune():
    config = Config(verbose=VERBOSE)
    will_remove = core.prune_list(config)
    for fpath in will_remove:
        print fpath.name
    response = click.confirm('Delete all the above wheels?')
    if response:
        for fpath in will_remove:
            fpath.unlink()
